% -----------------------------------------------------------------------------
% $Id: CString.lhs,v 1.9 2001/01/15 09:48:02 simonmar Exp $
%
% New interface: 
%   (c) The FFI task force, 2000
%
% DEPRECATED interface: 
%   (c) The GRASP/AQUA Project, Glasgow University, 1993-1998
%

C-specific marshalling support: string marshaling

Note: If you are looking for the routines on packed strings that resided here
formerly, refer to PackedString - the interface is much nicer anyway.


\begin{code}

module CString (

  -- representation of strings in C
  --
  CString,           -- = Ptr CChar
  CStringLen,        -- = (CString, Int)

  -- conversion of C strings into Haskell strings
  --
  peekCString,       -- :: CString    -> IO String
  peekCStringLen,    -- :: CStringLen -> IO String

  -- conversion of Haskell strings into C strings
  --
  newCString,        -- :: String -> IO CString
  newCStringLen,     -- :: String -> IO CStringLen

  -- conversion of Haskell strings into C strings using temporary storage
  --
  withCString,       -- :: String -> (CString    -> IO a) -> IO a
  withCStringLen,    -- :: String -> (CStringLen -> IO a) -> IO a

  -- conversion between Haskell and C characters *ignoring* the encoding
  --
  castCharToCChar,   -- :: Char -> CChar
  castCCharToChar,   -- :: CChar -> Char

  -- UnsafeCString: these might be more efficient than CStrings when
  -- passing the string to an "unsafe" foreign import.  NOTE: this
  -- feature might be removed in favour of a more general approach in
  -- the future.
  --
  UnsafeCString,     -- abstract
  withUnsafeCString, -- :: String -> (UnsafeCString -> IO a) -> IO a

  -- -------------------------------------------------------------------------
  -- chak: the following old interface is DEPRECATED, use PackedString instead
  -- -------------------------------------------------------------------------
    unpackCString       -- :: Addr -> [Char]
  , unpackCStringIO     -- :: Addr -> IO [Char]	
  , unpackCStringLenIO  -- :: Addr -> Int -> IO String
  , unpackCStringBA     -- :: ByteArray Int -> [Char]
  , unpackCStringBA#    -- :: ByteArray#    -> Int# -> [Char]
  , unpackCString#      -- :: Addr# -> [Char]

  , unpackNBytes        -- :: Addr -> Int -> [Char]
  , unpackNBytesST      -- :: Addr -> Int -> ST s [Char]
  , unpackNBytesAccST   -- :: Addr -> Int -> [Char] -> ST s [Char]
  , unpackNBytesAccIO   -- :: Addr -> Int -> [Char] -> IO [Char]
  , unpackNBytesBA      -- :: ByteArray Int -> Int  -> [Char]

  , unpackNBytes#       -- :: Addr# -> Int# -> [Char] **
  , unpackNBytesST#     -- :: Addr# -> Int# -> ST s [Char]
  , unpackNBytesBA#     -- :: ByteArray#    -> Int# -> [Char]

  , packString	        -- :: [Char] -> ByteArray Int
  , packStringIO	-- :: [Char] -> IO (ByteArray Int)
  , packStringST	-- :: [Char] -> ST s (ByteArray Int)
  , packNBytesST	-- :: Int -> [Char] -> ByteArray Int
  , packCString#	-- :: [Char] -> ByteArray#


    -- unmarshaling (char*) vectors.
  , unvectorize         -- :: Addr -> Int -> IO [String]
  , vectorize	        -- :: [[Char]] -> IO (ByteArray Int)


  , allocChars          -- :: Int -> IO (MutableByteArray RealWorld Int)
  , allocWords          -- :: Int -> IO (MutableByteArray RealWorld Int)
  , freeze	        -- :: MutableByteArray RealWorld Int -> IO (ByteArray Int)
  , strcpy	        -- :: Addr -> IO String
) where

import PrelCString 

-- imports for the DEPRECATED features
--
import PrelPtr
import PrelPack hiding	( unpackCString, unpackCStringST, 
			  unpackNBytes, unpackNBytesST, unpackNBytesAccST )
import GlaExts
import Addr		( Addr, nullAddr, readCharOffAddr, indexAddrOffAddr )
import PrelIOBase	( IO(..) )
\end{code}

% ----------------------------------------------------------------------------
% chak: Old DEPRECATED routines - do not use, they might just vanish
% ----------------------------------------------------------------------------

\begin{code}
unpackCString     (A# a)       = PrelPack.unpackCString     (Ptr a)
unpackCStringST   (A# a)       = PrelPack.unpackCStringST   (Ptr a)
unpackNBytes      (A# a) l     = PrelPack.unpackNBytes      (Ptr a) l
unpackNBytesST    (A# a) l     = PrelPack.unpackNBytesST    (Ptr a) l
unpackNBytesAccST (A# a) l acc = PrelPack.unpackNBytesAccST (Ptr a) l acc
\end{code}

\begin{code}
{-# DEPRECATED packStringIO "Use the module PackedString instead" #-}
packStringIO :: [Char] -> IO (ByteArray Int)
packStringIO str = stToIO (packStringST str)
\end{code}

NOTE: unpackCStringIO must traverse the entire string before
returning, since it is often used on dynamically allocated strings
which need to be deallocated after unpacking.

\begin{code}
{-# DEPRECATED unpackCStringIO "Use the module PackedString instead" #-}
unpackCStringIO :: Addr -> IO String
unpackCStringIO addr
 | addr == nullAddr = return "(null)"
 | otherwise        = unpack 0#
  where
    unpack nh = do
       ch <- readCharOffAddr addr (I# nh)
       if ch == '\0'
        then return []
        else do
         ls <- unpack (nh +# 1#)
         return (ch : ls)

{-# DEPRECATED unpackCStringLenIO "Use the module PackedString instead" #-}
-- unpack 'len' chars
unpackCStringLenIO :: Addr -> Int -> IO String
unpackCStringLenIO addr l@(I# len#)
 | len# <# 0#  = ioError (userError ("CString.unpackCStringLenIO: negative length (" ++ show l ++ ")"))
 | len# ==# 0# = return ""
 | otherwise   = unpack [] (len# -# 1#)
  where
    unpack acc 0# = do
       ch <- readCharOffAddr addr (I# 0#)
       return (ch:acc)
    unpack acc nh = do
       ch <- readCharOffAddr addr (I# nh)
       unpack (ch:acc) (nh -# 1#)

{-# DEPRECATED unpackNBytesAccIO "Use the module PackedString instead" #-}
unpackNBytesAccIO :: Addr -> Int -> [Char] -> IO [Char]
unpackNBytesAccIO (A# a) (I# len) acc = stToIO (unpackNBytesAccST# a len acc)
\end{code}

Turn a NULL-terminated vector of null-terminated strings into a string list
(ToDo: create a module of common marshaling functions)

\begin{code}
{-# DEPRECATED unvectorize "Use the module PackedString instead" #-}
unvectorize :: Addr -> Int -> IO [String]
unvectorize ptr n
  | str == nullAddr = return []
  | otherwise       = do
	x  <- unpackCStringIO str
	xs <- unvectorize ptr (n+1)
	return (x : xs)
  where
   str = indexAddrOffAddr ptr n

\end{code}

 Turn a string list into a NULL-terminated vector of null-terminated
strings No indices...I hate indices.  Death to Ix.

\begin{code}
{-# DEPRECATED vectorize "Use the module PackedString instead" #-}
vectorize :: [String] -> IO (ByteArray Int)
vectorize vs = do
  arr <- allocWords (len + 1)
  fill arr 0 vs
  freeze arr
 where
    len :: Int
    len = length vs

    fill :: MutableByteArray RealWorld Int -> Int -> [String] -> IO ()
    fill arr n [] =
	_casm_ ``((PP_)%0)[%1] = NULL;'' arr n
    fill arr n (x:xs) = do
	barr <- packStringIO x
        _casm_ ``((PP_)%0)[%1] = (P_)%2;'' arr n barr
	fill arr (n+1) xs

\end{code}

Allocating chunks of memory in the Haskell heap, leaving
out the bounds - use with care.

\begin{code}
{-# DEPRECATED allocChars "Use the module PackedString instead" #-}
-- Allocate a mutable array of characters with no indices.
allocChars :: Int -> IO (MutableByteArray RealWorld Int)
allocChars size = stToIO (newCharArray (0,size))

{-# DEPRECATED allocWords "Use the module PackedString instead" #-}
allocWords :: Int -> IO (MutableByteArray RealWorld Int)
allocWords size = stToIO (newIntArray (0,size))

{-# DEPRECATED freeze "Use the module PackedString instead" #-}
-- Freeze these index-free mutable arrays
freeze :: MutableByteArray RealWorld Int -> IO (ByteArray Int)
freeze mb = stToIO (unsafeFreezeByteArray mb)

{-# DEPRECATED strcpy "Use the module PackedString instead" #-}
-- Copy a null-terminated string from outside the heap to
-- Haskellized nonsense inside the heap
strcpy :: Addr -> IO String
strcpy str = unpackCStringIO str
\end{code}
