-----------------------------------------------------------------------------
--  $Id: IArray.hs,v 1.6 2001/03/30 04:54:28 qrczak Exp $
--
-- (c) The GHC Team 2000
--

module IArray ( 

	-- class of immutable array types
	IArray {- a:*->*->*  e:* -} (
	   array,  -- :: Ix ix => (ix,ix) -> [(ix,e)] -> a ix e	
	   (!),    -- :: Ix ix => a ix e -> ix -> e
	   (//)    -- :: Ix ix => a ix e -> [(ix,e)] -> a ix e
	),

	module Ix,
	HasBounds {- a:*->*->* -} (
	   bounds  -- :: Ix ix => a ix e -> (ix,ix)
	),

	-- non-strict polymorphic arrays
	Array.Array,

	-- strict, unboxed arrays
	UArray(..), -- non abstract because MArray needs to get at it

	assocs,    -- :: (Ix ix, IArray a e) => a ix e -> [(ix,e)]
	indices,   -- :: (Ix ix, HasBounds a) => a ix e -> [ix]
	amap,      -- :: (Ix ix, IArray a x, IArray a y) => 
		   --      (x -> y) -> a ix x -> a ix y
        listArray, -- :: (Ix ix, IArray a e) => (ix,ix) -> [e] -> a ix e
        elems,     -- :: (Ix ix, IArray a e) => a ix e -> [e]
	accum,     -- :: (Ix ix, IArray a e) => 
		   --	   (e->f->e) -> a ix e -> [(ix,f)] -> a ix e
        accumArray,-- :: (Ix ix, IArray a e) => 
		   --	   (e->f->e) -> e -> (ix,ix) -> [(ix,f)] -> a ix e
        ixmap      -- :: (Ix ix, Ix iy) => 
		   --	   (ix,ix) -> (ix->iy) -> a iy e -> a ix e
  ) where

import ArrayBase

-- exts
import ST

-- std
import List
import Ix
import qualified Array

-- prel
import PrelFloat
import PrelBase

-----------------------------------------------------------------------------
-- standard array ops

amap	   :: (Ix ix, IArray a x, IArray a y) => (x -> y) -> a ix x -> a ix y
amap f a   =  array b [(i, f (a!i)) | i <- range b]
           where b = bounds a

{-# SPECIALISE listArray :: (Int,Int) -> [b] -> Array.Array Int b #-}
listArray      :: (Ix ix, IArray a e) => (ix,ix) -> [e] -> a ix e
listArray b vs =  array b (zip (range b) vs)

{-# INLINE elems #-}
elems	   :: (Ix ix, IArray a e) => a ix e -> [e]
elems a    =  [a!i | i <- indices a]

ixmap	   :: (Ix ix, Ix iy, IArray a e)
	   => (ix,ix) -> (ix->iy) -> a iy e -> a ix e
ixmap b f a =  array b [(i, a ! f i) | i <- range b]

-----------------------------------------------------------------------------
-- Hacked-to-go-faster array ops

zap_with_f :: Ix ix => (a->b->a) -> STArray s ix a -> [(ix,b)] -> ST s ()
-- zap_with_f: reads an elem out first, then uses "f" on that and the new value
{-# INLINE zap_with_f #-}

zap_with_f f arr lst
  = foldr (zap_one f arr) (return ()) lst

zap_one f arr (i, new_v) rst = do
        old_v <- readArray arr i
	writeArray arr i (f old_v new_v)
	rst

accum  :: (Ix ix, IArray a e) => (e -> f -> e) -> a ix e -> [(ix,f)] -> a ix e
{-# INLINE accum #-}
accum f old_array ivs
  = runST (do
	-- copy the old array:
	arr <- thaw old_array
	-- now zap the elements in question with "f":
	zap_with_f f arr ivs
	unsafeFreeze arr
    )

accumArray :: (Ix ix, IArray a e)
	   => (e->f->e) -> e -> (ix,ix) -> [(ix,f)] -> a ix e
{-# INLINE accumArray #-}
accumArray f zero ixs ivs
  = runST (do
	arr <- newArray ixs zero
	zap_with_f f arr ivs
	unsafeFreeze arr
    )
